'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.connectWpQuery = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.rewind = rewind;
exports.identifier = identifier;
exports.wrapQueryFn = wrapQueryFn;
exports.connectWpPost = connectWpPost;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _lodash = require('lodash.get');

var _lodash2 = _interopRequireDefault(_lodash);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _isNodeFn = require('is-node-fn');

var _isNodeFn2 = _interopRequireDefault(_isNodeFn);

var _reactRedux = require('react-redux');

var _effects = require('redux-saga/effects');

var _debug = require('./util/debug');

var _debug2 = _interopRequireDefault(_debug);

var _contentTypesManager = require('./util/content-types-manager');

var _contentTypesManager2 = _interopRequireDefault(_contentTypesManager);

var _invariants = require('./invariants');

var _invariants2 = _interopRequireDefault(_invariants);

var _queryCounter = require('./util/query-counter');

var _queryCounter2 = _interopRequireDefault(_queryCounter);

var _findEntities = require('./util/find-entities');

var _findEntities2 = _interopRequireDefault(_findEntities);

var _actions = require('./redux/actions');

var _sagas = require('./redux/sagas');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

var WARN_NO_ENTITIES_PROP = 0;
var WARN_NO_REWIND = 1;

// Is a component the first Kasia component to mount?
var firstMount = true;

// What have we warned the consumer of?
var haveWarned = [];

/** Reset first mount flag, should be called before SSR of each request. */
function rewind() {
  firstMount = true;
}

/** Get entity identifier: either `id` as-is or the result of calling `id(props)`. */
function identifier(displayName, id, props) {
  var realId = typeof id === 'function' ? id(props) : id;
  _invariants2.default.isIdentifierValue(realId, displayName);
  return realId;
}

/** Wrap `queryFn` in a function that takes the node-wpapi instance. */
function wrapQueryFn(queryFn, props, state) {
  return (/*#__PURE__*/regeneratorRuntime.mark(function _callee(wpapi) {
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return (0, _effects.call)(queryFn, wpapi, props, state);

            case 2:
              return _context.abrupt('return', _context.sent);

            case 3:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    })
  );
}

/** Wrap component in react-redux connect. */
function connect(cls) {
  return (0, _reactRedux.connect)(function (_ref) {
    var wordpress = _ref.wordpress;

    _invariants2.default.hasWordpressObject(wordpress);
    return { wordpress: wordpress };
  })(cls);
}

var base = function base(target) {
  var _class, _temp;

  var displayName = target.displayName || target.name;

  return _temp = _class = function (_React$Component) {
    _inherits(KasiaConnectedComponent, _React$Component);

    function KasiaConnectedComponent() {
      _classCallCheck(this, KasiaConnectedComponent);

      return _possibleConstructorReturn(this, _React$Component.apply(this, arguments));
    }

    /** Make request for new data from WP-API. */
    KasiaConnectedComponent.prototype._requestWpData = function _requestWpData(props, queryId) {
      var action = this._getRequestWpDataAction(props);
      action.id = queryId;
      this.queryId = queryId;
      this.props.dispatch(action);
    };

    /** Find the query for this component and its corresponding data and return props object containing them. */


    KasiaConnectedComponent.prototype._reconcileWpData = function _reconcileWpData(props) {
      var query = this._query();
      var fallbackQuery = { complete: false, OK: null };

      var data = query && query.preserve ? query.result : this._makePropsData(props);

      if (query && query.error) {
        console.log('[kasia] error in query for ' + displayName + ':');
        console.log(query.error);
      }

      var result = {
        kasia: _defineProperty({
          query: query || fallbackQuery
        }, this.dataKey, data)
      };

      (0, _debug2.default)('content for ' + displayName + ' on `props.kasia.' + this.dataKey + '`');

      return Object.defineProperty(result, 'entities', {
        get: function get() {
          if (!haveWarned[WARN_NO_ENTITIES_PROP]) {
            console.log('[kasia] `props.kasia.entities` is replaced by `props.kasia.data` in v4.');
            haveWarned[WARN_NO_ENTITIES_PROP] = true;
          }
        }
      });
    };

    KasiaConnectedComponent.prototype._query = function _query() {
      return this.props.wordpress.queries[this.queryId];
    };

    KasiaConnectedComponent.prototype.componentWillMount = function componentWillMount() {
      var state = this.props.wordpress;
      var numQueries = Object.keys(state.queries).length;

      if (!numQueries && _queryCounter2.default.value > 0 && !haveWarned[WARN_NO_REWIND]) {
        console.log('[kasia] the query counter and queries in the store are not in sync. ' + 'This may be because you are not calling `kasia.rewind()` before running preloaders.');
        haveWarned[WARN_NO_REWIND] = true;
      }

      // When doing SSR we need to reset the counter so that components start
      // at queryId=0, aligned with the preloaders that have been run for them.
      if (firstMount) {
        _queryCounter2.default.reset();
        firstMount = false;
      }

      var queryId = this.queryId = _queryCounter2.default.next();
      var query = state.queries[queryId];

      // We found a prepared query matching `queryId` - use it.
      if (query && query.prepared) {
        (0, _debug2.default)('found prepared data for ' + displayName + ' at queryId=' + queryId);
      } else if (!(0, _isNodeFn2.default)()) {
        // No prepared query found
        if (!query) {
          // No query found, request data and reuse the queryId
          this._requestWpData(this.props, queryId);
        } else if (!query.prepared) {
          // Query found but it is not prepared, request data with new queryId
          this._requestWpData(this.props, _queryCounter2.default.next());
        }
      }
    };

    KasiaConnectedComponent.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
      var willUpdate = this._shouldUpdate(this.props, nextProps, this.context.store.getState());
      if (willUpdate) {
        (0, _debug2.default)(displayName, 'sending request for new data with props:', nextProps);
        this._requestWpData(nextProps, _queryCounter2.default.next());
      }
    };

    KasiaConnectedComponent.prototype.render = function render() {
      var props = Object.assign({}, this.props, this._reconcileWpData(this.props));
      return _react2.default.createElement(target, props);
    };

    return KasiaConnectedComponent;
  }(_react2.default.Component), _class.__kasia__ = true, _class.WrappedComponent = target, _class.contextTypes = {
    store: _propTypes2.default.object.isRequired }, _temp;
};

/**
 * Connect a component to a single entity from WordPress.
 *
 * @example Built-in content type, derived slug identifier:
 * ```js
 * const { Page } from 'kasia/types'
 * connectWordPress(Page, (props) => props.params.slug)(Component)
 * ```
 *
 * @example Built-in content type, explicit ID identifier:
 * ```js
 * const { Post } from 'kasia/types'
 * connectWordPress(Post, 16)(Component)
 * ```
 *
 * @example Custom content type, derived identifier:
 * ```js
 * connectWordPress('News', (props) => props.params.slug)(Component)
 * ```
 *
 * @param {String} contentType Content type of the WP entity to fetch
 * @param {Function|String|Number} id Entity's ID/slug/a function that derives either from props
 * @returns {Function} Decorated component
 */
function connectWpPost(contentType, id) {
  _invariants2.default.isString('contentType', contentType);
  _invariants2.default.isIdentifierArg(id);

  var typeConfig = _contentTypesManager2.default.get(contentType);

  return function (target) {
    var displayName = target.displayName || target.name;

    _invariants2.default.isNotWrapped(target, displayName);

    var KasiaConnectWpPostComponent = function (_base) {
      _inherits(KasiaConnectWpPostComponent, _base);

      function KasiaConnectWpPostComponent(props, context) {
        _classCallCheck(this, KasiaConnectWpPostComponent);

        var _this2 = _possibleConstructorReturn(this, _base.call(this, props, context));

        _this2.dataKey = contentType;
        return _this2;
      }

      KasiaConnectWpPostComponent.preload = function preload(props) {
        (0, _debug2.default)(displayName, 'connectWpPost preload with props:', props);
        _invariants2.default.isValidContentType(typeConfig, contentType, displayName + ' component');
        var action = (0, _actions.createPostRequest)(contentType, identifier(displayName, id, props));
        action.id = _queryCounter2.default.next();
        return [_sagas.fetch, action];
      };

      KasiaConnectWpPostComponent.prototype._getRequestWpDataAction = function _getRequestWpDataAction(props) {
        (0, _debug2.default)(displayName, 'connectWpPost request with props:', props);
        var realId = identifier(displayName, id, props);
        return (0, _actions.createPostRequest)(contentType, realId);
      };

      KasiaConnectWpPostComponent.prototype._makePropsData = function _makePropsData(props) {
        var query = this._query();

        if (!query || !query.complete || query.error) {
          return null;
        }

        if (query.preserve) {
          return query.result;
        }

        var entities = this.props.wordpress.entities[typeConfig.plural];

        if (entities) {
          var keys = Object.keys(entities);
          var realId = identifier(displayName, id, props);

          for (var i = 0, len = keys.length; i < len; i++) {
            var entity = entities[keys[i]];
            if (entity.id === realId || entity.slug === realId) {
              return entity;
            }
          }
        }

        return null;
      };

      KasiaConnectWpPostComponent.prototype._shouldUpdate = function _shouldUpdate(thisProps, nextProps) {
        // Make a request for new data if entity not in store or the identifier has changed
        var entity = this._makePropsData(nextProps);
        var identChanged = identifier(displayName, id, nextProps) !== identifier(displayName, id, thisProps);
        return !entity && identChanged;
      };

      KasiaConnectWpPostComponent.prototype.componentWillMount = function componentWillMount() {
        _invariants2.default.isValidContentType(typeConfig, contentType, displayName + ' component');
        _base.prototype.componentWillMount.call(this);
      };

      return KasiaConnectWpPostComponent;
    }(base(target));

    return connect(KasiaConnectWpPostComponent);
  };
}

/**
 * Connect a component to arbitrary data from WordPress.
 *
 * The component will request new data via the given `queryFn` if `shouldUpdate` returns true.
 *
 * @example Get all posts by an author:
 * ```js
 * connectWpQuery((wpapi) => wpapi.posts().embed().author('David Bowie'), () => true)
 * ```
 *
 * @example Get all pages:
 * ```js
 * connectWpQuery((wpapi) => wpapi.pages().embed(), () => true)
 * ```
 *
 * @example Get custom content type by slug (content type registered at init):
 * ```js
 * connectWpQuery(
 *   (wpapi) => wpapi.news().slug('gullible-removed-from-the-dictionary').embed(),
 *   () => true
 * )
 * ```
 *
 * @example Update only when `props.id` changes:
 * ```js
 * connectWpQuery((wpapi, props) => {
 *   return wpapi.page().id(props.identifier()).embed().get()
 * }, 'id')
 * ```
 *
 * @param {Function} queryFn Function that returns a wpapi query
 * @param {Function|String|Object} [shouldUpdate] Inspect props to determine if new data request is made
 * @param {Object} [opts] Options object
 * @param {Function|String|Object} [opts.shouldUpdate] See `shouldUpdate` docs
 * @param {Boolean} [opts.preserve] Preserve result of query and pass this untouched to component (no normalisation)
 * @returns {Function} Decorated component
 */
function connectWpQuery(queryFn, _shouldUpdate2) {
  var opts = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  if ((typeof _shouldUpdate2 === 'undefined' ? 'undefined' : _typeof(_shouldUpdate2)) === 'object') {
    opts = _shouldUpdate2;
    _shouldUpdate2 = opts.shouldUpdate;
  }

  _shouldUpdate2 = _shouldUpdate2 || function () {
    return false;
  };

  _invariants2.default.isObject('opts', opts);
  _invariants2.default.isFunction('queryFn', queryFn);
  _invariants2.default.isShouldUpdate(_shouldUpdate2);

  if (typeof _shouldUpdate2 === 'string') {
    _shouldUpdate2 = function shouldUpdate(thisProps, nextProps) {
      return (0, _lodash2.default)(thisProps, _shouldUpdate2) !== (0, _lodash2.default)(nextProps, _shouldUpdate2);
    };
  }

  return function (target) {
    var displayName = target.displayName || target.name;

    _invariants2.default.isNotWrapped(target, displayName);

    var KasiaConnectWpQueryComponent = function (_base2) {
      _inherits(KasiaConnectWpQueryComponent, _base2);

      function KasiaConnectWpQueryComponent(props, context) {
        _classCallCheck(this, KasiaConnectWpQueryComponent);

        var _this3 = _possibleConstructorReturn(this, _base2.call(this, props, context));

        _this3.dataKey = 'data';
        _this3._shouldUpdate = _shouldUpdate2;
        return _this3;
      }

      KasiaConnectWpQueryComponent.preload = function preload(props, state) {
        (0, _debug2.default)(displayName, 'connectWpQuery preload with props:', props, 'state:', state);
        var wrappedQueryFn = wrapQueryFn(queryFn, props, state);
        var action = (0, _actions.createQueryRequest)(wrappedQueryFn, opts.preserve);
        action.id = _queryCounter2.default.next();
        return [_sagas.fetch, action];
      };

      KasiaConnectWpQueryComponent.prototype._getRequestWpDataAction = function _getRequestWpDataAction(props) {
        (0, _debug2.default)(displayName, 'connectWpQuery request with props:', props);
        var wrappedQueryFn = wrapQueryFn(queryFn, props, this.context.store.getState());
        return (0, _actions.createQueryRequest)(wrappedQueryFn, opts.preserve);
      };

      KasiaConnectWpQueryComponent.prototype._makePropsData = function _makePropsData() {
        var query = this._query();
        var state = this.props.wordpress;
        if (!query || !query.complete || query.error) {
          return {};
        } else if (opts.preserve) {
          return query.result;
        } else {
          return (0, _findEntities2.default)(state.entities, state.keyEntitiesBy, query.entities);
        }
      };

      return KasiaConnectWpQueryComponent;
    }(base(target));

    return connect(KasiaConnectWpQueryComponent);
  };
}
exports.connectWpQuery = connectWpQuery;