'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.INITIAL_STATE = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.acknowledgeReducer = acknowledgeReducer;
exports.completeReducer = completeReducer;
exports.failReducer = failReducer;
exports.default = createReducer;

var _lodash = require('lodash.merge');

var _lodash2 = _interopRequireDefault(_lodash);

var _isNodeFn = require('is-node-fn');

var _isNodeFn2 = _interopRequireDefault(_isNodeFn);

var _pickEntityIds = require('../util/pick-entity-ids');

var _pickEntityIds2 = _interopRequireDefault(_pickEntityIds);

var _normalise = require('../util/normalise');

var _normalise2 = _interopRequireDefault(_normalise);

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var INITIAL_STATE = exports.INITIAL_STATE = {
  // WP-API request/response metadata are stored here
  queries: {},
  // Entities are normalised and stored here
  entities: {}

  /**
   * Merge all native and plugin reducers such that a single function reduces for a single action type.
   * @param {Object} reducers Plugin reducers
   * @param {Function} normaliser Function to normalise response data
   * @returns {Object} Reducer object
   */
};function mergeNativeAndThirdPartyReducers(reducers, normaliser) {
  var _baseReducer;

  var baseReducer = (_baseReducer = {}, _defineProperty(_baseReducer, _constants.ActionTypes.RequestAck, [acknowledgeReducer]), _defineProperty(_baseReducer, _constants.ActionTypes.RequestComplete, [completeReducer(normaliser)]), _defineProperty(_baseReducer, _constants.ActionTypes.RequestFail, [failReducer]), _baseReducer);

  // Group reducers by their action type
  var reducersByActionType = Object.keys(reducers).reduce(function groupByActionType(reducer, actionType) {
    reducer[actionType] = [].concat(reducer[actionType] || [], reducers[actionType] || []);
    return reducer;
  }, baseReducer);

  var reducer = {};

  // Produce a single function for each action type

  var _loop = function _loop(actionType) {
    if (reducersByActionType[actionType].length > 1) {
      // Call each reducer function in succession, passing the state returned from each to the next
      reducer[actionType] = function (state, action) {
        return reducersByActionType[actionType].reduce(function (state, fn) {
          return fn(state, action);
        }, state);
      };
    } else {
      // Take the first and only function as the whole reducer
      reducer[actionType] = reducersByActionType[actionType][0];
    }
  };

  for (var actionType in reducersByActionType) {
    _loop(actionType);
  }

  return reducer;
}

// ACKNOWLEDGE
// Place record of request on store
function acknowledgeReducer(state, action) {
  return (0, _lodash2.default)({}, state, {
    queries: _defineProperty({}, action.id, {
      id: action.id,
      prepared: (0, _isNodeFn2.default)(),
      complete: false,
      OK: null
    })
  });
}

// COMPLETE
// Place entity on the store; update query record if for component (has an id)
function completeReducer(normalise) {
  return function (state_, action) {
    var state = (0, _lodash2.default)({}, state_);
    var query = state.queries[action.id];

    // action.id === null when created via util/preloadQuery
    if (action.id !== null && !query) {
      throw new Error('cannot complete non-existent query');
    }

    state.entities = (0, _lodash2.default)(state.entities, normalise(action.data));

    // The action id would be null if the preloadQuery method has initiated
    // the completeRequest action as they do not need a query in the store
    // (there is no component to pick it up).
    if (typeof action.id === 'number') {
      var updated = {
        id: action.id,
        prepared: (0, _isNodeFn2.default)(),
        complete: true,
        OK: true
      };

      if (query.preserve) {
        updated.result = action.data;
        updated.paging = null;
      } else {
        updated.entities = (0, _pickEntityIds2.default)(action.data);
        updated.paging = action.data._paging || {};
      }

      state.queries[action.id] = updated;
    }

    return state;
  };
}

// FAIL
// Update query record only
function failReducer(state, action) {
  return (0, _lodash2.default)({}, state, {
    queries: _defineProperty({}, action.id, {
      id: action.id,
      error: action.error,
      prepared: (0, _isNodeFn2.default)(),
      complete: true,
      OK: false
    })
  });
}

/**
 * Create the aggregate reducer for an instance of Kasia.
 * @param {Object} keyEntitiesBy Entity property used as key in store
 * @param {Object} reducers Plugin reducers
 * @returns {Object} Kasia reducer
 */
function createReducer(_ref) {
  var keyEntitiesBy = _ref.keyEntitiesBy,
      reducers = _ref.reducers;

  var normaliser = function normaliser(data) {
    return (0, _normalise2.default)(data, keyEntitiesBy);
  };
  var reducer = mergeNativeAndThirdPartyReducers(reducers, normaliser);
  var initialState = Object.assign({}, INITIAL_STATE, { keyEntitiesBy: keyEntitiesBy });

  return {
    wordpress: function kasiaReducer() {
      var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
      var action = arguments[1];

      var _action$type$split = action.type.split('/'),
          _action$type$split2 = _slicedToArray(_action$type$split, 1),
          actionNamespace = _action$type$split2[0];

      if (actionNamespace === 'kasia' && action.type in reducer) {
        return reducer[action.type](state, action);
      }

      return state;
    }
  };
}