'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fetch = fetch;
exports.watchRequests = watchRequests;

var _effects = require('redux-saga/effects');

var _wpapi = require('../wpapi');

var _wpapi2 = _interopRequireDefault(_wpapi);

var _constants = require('../constants');

var _actions = require('./actions');

var _queryBuilder = require('../util/query-builder');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _marked = /*#__PURE__*/regeneratorRuntime.mark(fetch),
    _marked2 = /*#__PURE__*/regeneratorRuntime.mark(watchRequests);

/**
 * Make a fetch request to the WP-API according to the action
 * object and record the result in the store.
 * @param {Object} action Action object
 */
function fetch(action) {
  var wpapi, fn, data;
  return regeneratorRuntime.wrap(function fetch$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.prev = 0;
          _context.next = 3;
          return (0, _effects.put)((0, _actions.acknowledgeRequest)(action));

        case 3:
          wpapi = (0, _wpapi2.default)();
          fn = action.queryFn || (0, _queryBuilder.buildQueryFunction)(action);
          _context.next = 7;
          return (0, _effects.call)(fn, wpapi);

        case 7:
          data = _context.sent;
          _context.next = 10;
          return (0, _effects.put)((0, _actions.completeRequest)(action.id, data));

        case 10:
          _context.next = 16;
          break;

        case 12:
          _context.prev = 12;
          _context.t0 = _context['catch'](0);
          _context.next = 16;
          return (0, _effects.put)((0, _actions.failRequest)(action.id, _context.t0.stack || _context.t0.message));

        case 16:
        case 'end':
          return _context.stop();
      }
    }
  }, _marked, this, [[0, 12]]);
}

/** Watch request create actions and fetch data for them. */
function watchRequests() {
  return regeneratorRuntime.wrap(function watchRequests$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          _context2.next = 2;
          return (0, _effects.takeEvery)([_constants.ActionTypes.RequestCreatePost, _constants.ActionTypes.RequestCreateQuery], fetch);

        case 2:
        case 'end':
          return _context2.stop();
      }
    }
  }, _marked2, this);
}